/*************************************************************************************
 Data Visualization - Basic 3D Demo                
 Tested in Processing 3                                    
 
 Author: Rodger (Jieliang) Luo 
 Re-edit: Weidi Zhang
 Supervisor: George Legrady                           
 
 Purpose: Show how to present volume data in a basic 3D environment     
 
 Usage: 1. A mouse left-drag will rotate the camera around the subject.
 2. A right drag will zoom in and out. 
 3. A middle-drag (command-left-drag on mac) will pan. 
 4. A double-click restores the camera to its original position. 
 5. The shift key constrains rotation and panning to one axis or the other.
 *************************************************************************************/


import peasy.*;
PeasyCam cam;
Table table;
int numRows;
int startYear =2006, endYear = 2015;

//declare the arraylist to fill the arraylist with book objects
ArrayList<Book>books = new ArrayList<Book>();

void setup() {
  size(800, 800, P3D);

  // constructors - PeasyCam [downloaded library]
  // PeasyCam(PApplet parent, double distance);
  // PeasayCame(Papplet parent, double lookAtX, double lookAtY, double lookAtZ, double distance)
  cam = new PeasyCam(this, 300);

  // step 1 . load table and get the basic information of data
  table = loadTable("Elements2.csv", "header");
  numRows = table.getRowCount();
  println("Rows : " + numRows);

  // step 2. add object
  for (int i =0; i<numRows; i++) {
    String element = table.getString(i, 0);
    String time = table.getString(i, 1);
    String date = table.getString(i, 2);
    int count = table.getInt(i, 3);
    String title = table.getString(i, 4);
    books.add(new Book(element, time, date, count, title));
  }
}


void draw() {
  background(50);
  drawData();
  drawLabel();
  noStroke();
  fill(200, 20);
  box(100, 100, 100);
}

void drawData() {
  // X axis - year
  // Y axis - time
  // Z axis - count
  for ( int i = 0; i< numRows; i++) {
    // Year 2006 - 2015 
    float yearAxis = map(dateParser(books.get(i).date), 0, 3650, -50, 50);
    // time 10AM - 8PM
    float timeAxis = map(timeParser(books.get(i).time), 600, 1200, -50, 50);
    // count 
    float countAxis = map(log(books.get(i).count), 0, log(12557), -50, 50);
    // user interaction - keyPressed
    //push and pop - save the current transformation state
    pushMatrix(); //save
    // translate - moves the point of origin
    translate(yearAxis, timeAxis, countAxis);
    strokeWeight(3);
    if (fire) {
      if (books.get(i).element.equals("fire") ||books.get(i).element.equals("Fire")) {
        stroke(247, 95, 0);
        point(0, 0, 0);
      }
    } 
    if (wood) {
      if (books.get(i).element.equals("wood") ||books.get(i).element.equals("Wood")) {
        stroke(0, 188, 86);
        point(0, 0, 0);
      }
    }

    if (water) {
      if (books.get(i).element.equals("water") ||books.get(i).element.equals("Water")) {
        stroke(40, 164, 245);
        point(0, 0, 0);
      }
    }
    if (gold) {
      if (books.get(i).element.equals("gold") ||books.get(i).element.equals("Gold")) {
        stroke(247, 194, 0);
        point(0, 0, 0);
      }
    } 

    popMatrix(); //restore / undo
  }
}


void drawLabel() {
  fill(255);
  textSize(30);
  // X axis - year
  pushMatrix();
  textAlign(LEFT, TOP);
  translate(-50, 50, 50);
  scale(0.1);
  text(startYear, 0, 0, 0);
  popMatrix();

  pushMatrix();
  textAlign(RIGHT, TOP);
  translate(50, 50, 50);
  scale(0.1);
  text(endYear, 0, 0, 0);
  popMatrix();

  //Y-axis - time
  pushMatrix();
  textAlign(RIGHT, TOP);
  translate(-50, 50, 50);
  scale(0.1);
  text("10AM", 0, 0, 0);
  popMatrix();

  pushMatrix();
  textAlign(RIGHT, TOP);
  translate(-50, -50, 50);
  scale(0.1);
  text("8PM", 0, 0, 0);
  popMatrix();

  //Z-axis - count
  pushMatrix();
  textAlign(LEFT, BOTTOM);
  translate(50, 50, -50);
  scale(0.1);
  text("COUNT 0", 0, 0, 0);
  popMatrix();

  pushMatrix();
  textAlign(LEFT, BOTTOM);
  translate(50, 50, 50);
  scale(0.1);
  text("COUNT 12557", 0, 0, 0);
  popMatrix();
}
